﻿import reefling.classes.util.*
class Rotator extends AdvObj{
	
	var radius:Number;				//the radius of the circle made by the rotatorobject
	var controlled:AdvObj;			//the object to rotate
	var pressed:Boolean;			//checks if the rotator is pressed
	var attached:Boolean;			//is the rotator attached to the object to control or not?
	var startAngle:Number;			//the start angle the mouse pointer will indicate
	var oriAngle:Number;			//original angle of controlled object before change
	
	function Rotator(movie:MovieClip, id:String, ctrl:AdvObj, r:Number){
		super(movie, id);
		
		this.controlled = ctrl;
		this.pressed = false;
		this.startAngle = null;
		this.oriAngle = null;
		
		//if a radius bigger than 0 is passed the rotator is considered unattached, otherwise the initial
		//distance between the controlpoints is used
		if(r > 0){
			this.radius = r;
			this.attached = false;
		}else{
			this.radius = HSMath.d2abs(this.controlled.ctrlPoint, this.ctrlPoint);
			this.ctrlPoint.setVpos(this.controlled.ctrlPoint);
			this.attached = true;
		}
		
		//the init-object initiates some callback functions
		HSInit.initRotator(this);
	}
	
	function rotate(){
		
		var xdif:Number = 0;
		var ydif:Number = 0;
		
		//first we calculate the difference between the mouse pointer and the controlled object's controlpoint
		//if the rotator isn't attached we use it's own controlpoint as reference
		if(this.attached){
			xdif = _xmouse - this.controlled.ctrlPoint.x;
			ydif = _ymouse - this.controlled.ctrlPoint.y;
		}else{
			xdif = _xmouse - this.ctrlPoint.x;
			ydif = _ymouse - this.ctrlPoint.y;
		}
		
		//the angle is calculated with the help of the differences and the difference between current
		//"mouse angle" and the startAngle first calculated when we pressed the button
		this.angle = this.oriAngle + (Math.atan2(ydif, xdif) - this.startAngle);
		//the location of the current point is set using the angle and the controlpoint
		this.curPoint.set(this.ctrlPoint.x + (Math.cos(this.angle)*this.radius), 
						  this.ctrlPoint.y + (Math.sin(this.angle)*this.radius));
		//the rotator itself is rotated
		super.rotate();
		//the controlled object gets the same angle as the rotator and is rotated
		this.controlled.angle = this.angle;
		this.controlled.rotate();
	}
	
	//overriding the super's version, if the rotator is attached it will follow the controlled object,
	//otherwise it will just rotate if pressed
	function drawMe(){
		if(this.attached)
			super.setMe(this.controlled.curPoint);

		if(this.pressed){
			if(this.startAngle == null){
				this.startAngle = Math.atan2(_ymouse - this.controlled.ctrlPoint.y, _xmouse - this.controlled.ctrlPoint.x);
				this.oriAngle = this.angle;
			}
			this.rotate();
		}else
			this.startAngle = null;
				
		super.drawMe();
	}
}