﻿import reefling.classes.util.*
class ShoppingCart{
	
	var cartItems:Array;				//array containing the items in the cart
	var cartheight:Number;				//height of cart in pixels
	var pMovie:MovieClip;				//parent movieclip
	var numItems:Number;				//number of items in cart (used to save space)
	var shipping:Number;				//the shipping cost
	var currentCurrency:String;		//the used currency
	var currencyMultiplier:Number;		//the used currency's value against the euro
	var currencies:Currencies;			//a list of currencies and their values against the euro
	var payed:Boolean;					//is the cart payed for?
	var emsq:Boolean;					//is shipping by ems available?
	var ems:Array;						//an array of shipping costs for ems at different weights
	var air:Array;						//an array of shipping costs for normal air shipping at different weights
	var country:String;
	var discount:Number;
	var initWeight:Number;
	
	function ShoppingCart(p:MovieClip, shipping:Number){
		this.cartheight = 0;
		this.pMovie = p;
		this.numItems = 0;
		this.shipping = shipping;				//initial shippingcost will be set to 0
		this.cartItems = new Array();
		this.currentCurrency = "EUR";			//the cart starts with euro
		this.currencyMultiplier = 1;			//which is multiplied by 1 ofcourse
		this.currencies = new Currencies();
		this.payed = false;
		this.emsq = false;						//we start with air
		this.ems = new Array(0, 0, 0, 0, 0, 0, 0, 0, 0); //all shippingcosts are 0
		this.air = new Array(0, 0, 0, 0, 0, 0, 0, 0, 0);
		this.currencies.addCurrency("EUR", 1);		//initial numbers to avoid NaNs initially
		this.currencies.addCurrency("USD", 1.3);
		this.currencies.addCurrency("THB", 47);
		this.calcTotal();
		this.country = "0";
		this.discount = 0;
		this.initWeight = 0;
	}
	
	function addCartItem(itemLabel:String, weight:Number, price:Number, stock:Number, code:String){
		
		var inCart:Boolean = false;
		for(var i:Number = 0; i < this.numItems; i++){
			if(this.cartItems[i].cont.label_txt.text == itemLabel){
				//if(Number(this.cartItems[i].cont.amount_txt.text) < stock){
					this.cartItems[i].amount++;
					this.cartItems[i].curWeight = this.cartItems[i].amount * this.cartItems[i].orgWeight;
					this.cartItems[i].cont.amount_txt.text = this.cartItems[i].amount;
					this.cartItems[i].cont.price_txt.text = Number(this.cartItems[i].cont.amount_txt.text) * this.cartItems[i].price * this.currencyMultiplier;
					this.calcTotal();
					this.updateCreateOrder();
				//}
				
				inCart = true;
			}
		}
		
		if(!inCart){
			var temp:CartItem = new CartItem(this.pMovie, weight, this.numItems, code, price, itemLabel, this);
			this.cartItems.push(temp);
			this.cartItems[i].cont.price_txt.text = this.cartItems[i].price * this.currencyMultiplier;
			this.numItems++;
			this.update();
			this.updateCreateOrder();
		}
	}
	
	//calculation of total weight
	function calcWeight():Number{
		var temp:Number = this.initWeight;
		for(var i:Number = 0; i < this.numItems; i++){
			temp += this.cartItems[i].curWeight;
		}
		
		if(temp > 4500) 
			temp = 0;

		this.pMovie.weight_mc.weight_txt.text = temp;
		return temp;
	}
	
	//set shippingcost based on chosen country
	function setShipping(ems:String, air:String, shipping_addition:Number){
		if(ems.length < 2){
			HSHelp.setStringArray("na", this.ems);
			this.air = air.split(";");
			HSHelp.addNumberToAllElements(shipping_addition, this.air);
			this.emsq = false;
		}else{
			this.ems = ems.split(";");
			HSHelp.addNumberToAllElements(shipping_addition, this.ems);
			this.air = air.split(";");
			HSHelp.addNumberToAllElements(shipping_addition, this.air);
			
		}
		//trace(this.ems[0]);
		this.calcTotal();
		
	}
	
	//calculate shipping with currently stored values
	function calcShipping(){
		var weight:Number = this.calcWeight();
		if(this.emsq)
			this.shipping = (this.ems[HSHelp.getPosFromInterval(weight, 500)] * this.currencyMultiplier) / this.currencies.getExchRate("THB");
		else
			this.shipping = (this.air[HSHelp.getPosFromInterval(weight, 500)] * this.currencyMultiplier) / this.currencies.getExchRate("THB");
			
		
		if(weight == 0) 
			this.shipping = 0;
			
		this.pMovie.shipping_mc.shipping_txt.text = this.shipping;
	}
	
	function calcDiscount(subtotal:Number):Number{
		subtotal /= this.currencyMultiplier;
		
		if(subtotal > 150)
			return 0.15;
		
		return subtotal/1000;
			
		
		
	}
	//calculate subtotal of goods only
	function calcSubTotal(){
		var total:Number = 0;
		for(var i:Number = 0; i < this.numItems; i++){
			var temp:String = this.cartItems[i].cont.price_txt.text;
			total += Number(this.cartItems[i].cont.price_txt.text);
			HSHelp.setTextNbrDecimals(this.cartItems[i].cont.price_txt, 2, ".");
		}
		this.discount = this.calcDiscount(total);
		total -= total * this.discount;
		this.pMovie.discount_mc.discount_txt.text = 100*this.discount;
		HSHelp.setTextNbrDecimals(this.pMovie.discount_mc.discount_txt, 0, ".");
		this.pMovie.subtotal_mc.subtotal_txt.text = total;
		this.pMovie.discount_mc.discount_txt.text += "%";
	}
	
	function setDecimals(n:Number){
		HSHelp.setTextNbrDecimals(this.pMovie.total_mc.total_txt, n, ".");
		HSHelp.setTextNbrDecimals(this.pMovie.subtotal_mc.subtotal_txt, n, ".");
		HSHelp.setTextNbrDecimals(this.pMovie.shipping_mc.shipping_txt, n, ".");
	}
	
	//calculate both cost of merchandise and shipping
	function calcTotal(){
		this.calcSubTotal();
		this.calcShipping();
		var total:Number = 0;
		total += Number(this.pMovie.shipping_mc.shipping_txt.text);
		total += Number(this.pMovie.subtotal_mc.subtotal_txt.text);
		this.pMovie.total_mc.total_txt.text = total;
		this.setDecimals(3);
	}
	
	//invoked when user choses currency also invoked initially for the default values
	function handleRadioExclSel(value:String){
		if(value == "EUR" || value == "USD"){
			this.currentCurrency = value;
			this.currencyMultiplier = this.currencies.getExchRate(value);
			for(var i:Number = 0; i < this.numItems; i++)
				this.cartItems[i].cont.price_txt.text = this.cartItems[i].amount * this.cartItems[i].price * this.currencyMultiplier;
		}
		
		if(value == "EMS")
			this.emsq = true;
		
		if(value == "AIR")
			this.emsq = false;
		
		this.calcTotal();
	}
	
	function getShippingOption():String{
		if(this.emsq)
			return "EMS";
		else
			return "AIR";
	}
	
	//returns the total cost of cart including shipping
	function getTotal():String{
		return this.pMovie.total_mc.total_txt.text;
	}
	
	function getSubTotal():String{
		return this.pMovie.subtotal_mc.subtotal_txt.text;
	}
	
	function getSubTotalAsNumber():Number{
		return Number(this.pMovie.subtotal_mc.subtotal_txt.text);
	}
	
	function getShipping():String{
		return this.pMovie.shipping_mc.shipping_txt.text;
	}
	
	//removes an item at a certain position
	function removeItems(pos:Number){
		//if this position contains more than one item we only remove one item
		if(this.cartItems[pos].amount > 1){
			this.cartItems[pos].cont.amount_txt.text = Number(this.cartItems[pos].cont.amount_txt.text) - 1;
			this.cartItems[pos].amount--;
			this.cartItems[pos].cont.price_txt.text = Number(this.cartItems[pos].cont.amount_txt.text) * this.cartItems[pos].price;
			this.cartItems[pos].curWeight = this.cartItems[pos].amount * this.cartItems[pos].orgWeight;
		//otherwise we remove the whole position as all items will be removed from it
		}else{
			this.cartItems[pos].cont.removeMovieClip();
			this.cartItems.splice(pos, 1);
			this.numItems--;
		}
		this.updateCreateOrder();
		this.update();
	}
	
	//send the items to paypal when user presses paybutton
	function itemsToPaypal(order_status:String):String{
		var temp:String = "";
		var amounts:String = "";
		var itemcodes:String = "";
		
		for(var i:Number = 0; i < this.numItems; i++){
			temp += this.cartItems[i].cont.label_txt.text+" ";
			amounts += this.cartItems[i].amount+";";
			itemcodes += this.cartItems[i].code+";";
			if(i != this.numItems -1)
				temp += this.cartItems[i].cont.amount_txt.text+"pcs and ";
			else
				temp += this.cartItems[i].cont.amount_txt.text+"pcs.";
		}
		
		var send_data:LoadVars = new LoadVars();
		send_data.itemcodes = itemcodes;
		send_data.amounts = amounts;
		send_data.invoice = _root.invoice;
		send_data.country = this.country;
		send_data.currency = this.currentCurrency;
		send_data.subtotal = this.getSubTotal();
		send_data.shipping = this.getShipping();
		send_data.total = this.getTotal();
		send_data.status = order_status;
		
		MysqlConnect.sendToPhp(send_data, _root.path+"order_store.php");
		
		this.payed = true;
		return temp;
	}
	
	//updates the cart if a position becomes deleted completely
	function update(){
		this.cartheight = 0;
		for(var i:Number = 0; i < this.numItems; i++){
			this.cartItems[i].pos = i;
			this.cartItems[i].cont._y = this.cartheight;
			this.cartheight += this.cartItems[i].cont._height;
		}
		this.pMovie.subtotal_mc._y = this.cartheight;
		this.pMovie.discount_mc._y = this.cartheight;
		this.pMovie.shipping_mc._y = this.cartheight + this.pMovie.subtotal_mc._height;
		this.pMovie.weight_mc._y = this.cartheight + this.pMovie.subtotal_mc._height;
		this.pMovie.total_mc._y = this.cartheight + this.pMovie.subtotal_mc._height + this.pMovie.shipping_mc._height;
		this.calcTotal();
	}
	
	//when the user cancels the purchase by going somewhere else on the site
	function returnStuff(){
		var send_data:LoadVars = new LoadVars();
		send_data.invoice = _root.invoice;
		MysqlConnect.sendToPhp(send_data, _root.path+"order_delete.php");
	}

	//creates an order with current invoice number if no one exists in the database, otherwise the 
	//current order is updated
	function updateCreateOrder(){
		var amounts:String = "";
		var itemcodes:String = "";
		for(var i:Number = 0; i < this.numItems; i++){
			amounts += this.cartItems[i].amount+";";
			itemcodes += this.cartItems[i].code+";";
		}
		var send_data:LoadVars = new LoadVars();
		send_data.itemcodes = itemcodes;
		send_data.amounts = amounts;
		send_data.invoice = _root.invoice;
		send_data.country = this.country;
		MysqlConnect.sendToPhp(send_data, _root.path+"order_create_update.php");
	}
}