﻿import reefling.classes.util.*
class Surf{
	
	var cPoint:V2D;				//the controlpoint that the movieclip will follow
	var strength:V2D;			//the strength of the oscillation
	var eccentricity:Number;	//randomness of frequency
	var speed:Number;			//the general speed of the motion
	//var minMaxAngle:V2D;		//minimum and maximum oscillation angle in 2D. (not implemented yet)
	var movie:MovieClip;		//the affected movie
	var angle:V2D;				//angles used for translation
	var oPoint:V2D;				//the startpoint for the motion which will act as a reference
	var cSpeed:V2D;				//speed of cPoint motion in 2 dim.
	var cStrength:V2D;			//amplitude of motion in 2 dim.
	
	function Surf(m:MovieClip, stX:Number, stY:Number, e:Number, sp:Number, minAng:Number, maxAng:Number){
		this.cPoint 		= new V2D(0,0);
		this.strength 		= new V2D(stX,stY);
		this.eccentricity 	= e;
		this.speed 			= sp;
		//this.minMaxAngle 	= new V2D(minAng, maxAng);
		this.movie			= m;
		this.angle			= new V2D(0,0);
		this.oPoint			= new V2D(m._x, m._y);
		this.cStrength		= new V2D(0,0);
		this.cSpeed			= new V2D(this.speed,this.speed);
		//if the movie is draggable it must implement certain functionality to prevent the Surfmotion from
		//setting the position, the main thing is a flag which will tell the surf if the movie is being dragged
		this.movie.handleDrag = false;
	}
	
	function connectToAnimal(a:Animal){
		this.oPoint = a.cPoint;
	}
	
	//resets the surf, happens if the target movie is being dragged
	function reset(){
		this.oPoint.movieSet(this.movie);
		this.cPoint.set(0,0);
		this.angle.set(0,0);
		this.cSpeed.set(this.speed,this.speed);
		this.cStrength.set(0,0);
	}
	
	function move(){
		//if the movie is not being dragged, translate it, otherwise reset it
		if(!this.movie.handleDrag){
			//if the angle is bigger than one revolution or zero, start a new surf 
			//all surfs will be different
			if(this.angle.x > Math.abs(HSMath.PItimes2) || this.angle.x == 0){
				this.angle.x = 0;
				this.cStrength.x = this.strength.x + (Math.random() * this.eccentricity);
				this.cSpeed.x = this.speed * (Math.random()*this.eccentricity);
				//if(Math.random() < 0.5)
					//this.cSpeed.x = -this.cSpeed.x;
			}
				
			if(this.angle.y > Math.abs(HSMath.PItimes2) || this.angle.y == 0){
				this.angle.y = 0;
				this.cStrength.y = this.strength.y + (Math.random() * this.eccentricity);
				this.cSpeed.y = this.speed * (Math.random()*this.eccentricity);
			}
			
			//first the angle is being increased by the speed
			this.angle.y += HSMath.HSRAD * this.cSpeed.y;
			this.angle.x += HSMath.HSRAD * this.cSpeed.x;
			//then the controlpoint is being translated with the angle and the amplitude
			this.cPoint.y = Math.sin(this.angle.y) * this.cStrength.y;
			this.cPoint.x = Math.sin(this.angle.x) * this.cStrength.x;
			//lastly the movie is being translated relative to the original point
			this.movie._y = oPoint.y + cPoint.y;
			this.movie._x = oPoint.x + cPoint.x;
		}else 
			this.reset();
	}
}